package com.jarvanmo.rammus

import android.content.pm.ApplicationInfo
import android.os.Bundle
import android.net.Uri
import android.app.Application
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.os.Looper
import com.alibaba.sdk.android.push.CommonCallback
import com.alibaba.sdk.android.push.HonorRegister
import com.alibaba.sdk.android.push.huawei.HuaWeiRegister
import com.alibaba.sdk.android.push.noonesdk.PushServiceFactory
import com.alibaba.sdk.android.push.register.*
import com.blankj.utilcode.util.LogUtils
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


class RammusPlugin : FlutterPlugin, MethodCallHandler {
    companion object {
        private val inHandler = Handler(Looper.getMainLooper())
        lateinit var gottenApplication: Context


    }

    private lateinit var channel: MethodChannel

    override fun onMethodCall(call: MethodCall, result: Result) {
        LogUtils.d(call.method)
        when (call.method) {
            "register" -> register()
            "initPushService" -> initPushService()
            "deviceId" -> result.success(PushServiceFactory.getCloudPushService().deviceId)
            "turnOnPushChannel" -> turnOnPushChannel(result)
            "turnOffPushChannel" -> turnOffPushChannel(result)
            "checkPushChannelStatus" -> checkPushChannelStatus(result)
            "bindAccount" -> bindAccount(call, result)
            "unbindAccount" -> unbindAccount(result)
            "bindTag" -> bindTag(call, result)
            "unbindTag" -> unbindTag(call, result)
            "listTags" -> listTags(call, result)
            "addAlias" -> addAlias(call, result)
            "removeAlias" -> removeAlias(call, result)
            "listAliases" -> listAliases(result)
            "setupNotificationManager" -> setupNotificationManager(call, result)
            "bindPhoneNumber" -> bindPhoneNumber(call, result)
            "unbindPhoneNumber" -> unbindPhoneNumber(result)
            "applicationBadgeNumberClean" -> setApplicationBadgeNumber(call)
            else -> result.notImplemented()
        }

    }

    private fun register() {

        val pushService = PushServiceFactory.getCloudPushService()
        pushService.register(gottenApplication, object : CommonCallback {
            override fun onSuccess(response: String?) {
                inHandler.postDelayed({
                    RammusPushHandler.methodChannel?.invokeMethod(
                        "initCloudChannelResult", mapOf(
                            "isSuccessful" to true,
                            "response" to response
                        )
                    )
                }, 2000)
            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                inHandler.postDelayed({
                    RammusPushHandler.methodChannel?.invokeMethod(
                        "initCloudChannelResult", mapOf(
                            "isSuccessful" to false,
                            "errorCode" to errorCode,
                            "errorMessage" to errorMessage
                        )
                    )
                }, 2000)
            }
        })
        val appInfo = gottenApplication.packageManager
            .getApplicationInfo(gottenApplication.packageName, PackageManager.GET_META_DATA)
        val xiaomiAppId = appInfo.metaData.getString("com.xiaomi.push.client.app_id")
        val xiaomiAppKey = appInfo.metaData.getString("com.xiaomi.push.client.app_key")
        LogUtils.d("小米：appId=$xiaomiAppId,appKey=$xiaomiAppKey")
        if ((xiaomiAppId != null && xiaomiAppId.isNotBlank())
            && (xiaomiAppKey != null && xiaomiAppKey.isNotBlank())
        ) {
            LogUtils.d("正在注册小米推送服务...")
            MiPushRegister.register(gottenApplication, xiaomiAppId, xiaomiAppKey)
        }
        val huaweiAppId = appInfo.metaData.getString("com.huawei.hms.client.appid")
        LogUtils.d("华为：appId=$huaweiAppId")
        if (huaweiAppId != null && huaweiAppId.toString().isNotBlank()) {
            LogUtils.d("正在注册华为推送服务...")
            HuaWeiRegister.register(gottenApplication as Application)
        }
        val oppoAppKey = appInfo.metaData.getString("com.oppo.push.client.app_key")
        val oppoAppSecret = appInfo.metaData.getString("com.oppo.push.client.app_secret")
        LogUtils.d("oppo：oppoAppKey=$oppoAppKey,oppoAppSecret=$oppoAppSecret")
        if ((oppoAppKey != null && oppoAppKey.isNotBlank())
            && (oppoAppSecret != null && oppoAppSecret.isNotBlank())
        ) {
            LogUtils.d("正在注册Oppo推送服务...")
            OppoRegister.register(gottenApplication, oppoAppKey, oppoAppSecret)
        }
        val vivoAppId = appInfo.metaData.getString("com.vivo.push.app_id")
        val vivoApiKey = appInfo.metaData.getString("com.vivo.push.api_key")
        LogUtils.d("vivo：vivoAppId=$vivoAppId,vivoApiKey=$vivoApiKey")
        if ((vivoAppId != null && vivoAppId.isNotBlank())
            && (vivoApiKey != null && vivoApiKey.isNotBlank())
        ) {
            LogUtils.d("正在注册vivo推送服务...")
            VivoRegister.register(gottenApplication)
        }
        val hihonorId = appInfo.metaData.getString("com.hihonor.push.app_id")
        LogUtils.d("荣耀：hihonorId=$hihonorId")
        if (hihonorId != null && hihonorId.toString().isNotBlank()) {
            LogUtils.d("正在注册荣耀推送服务...")
            HonorRegister.register(gottenApplication as Application)
        }
    }

   private  fun initPushService(){
       PushServiceFactory.init(gottenApplication)
       val pushService = PushServiceFactory.getCloudPushService()
       pushService.setPushIntentService(RammusPushIntentService::class.java)
       LogUtils.d("initPushService")
   }


    private fun turnOnPushChannel(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.turnOnPushChannel(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }

    private fun turnOffPushChannel(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.turnOffPushChannel(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun checkPushChannelStatus(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.checkPushChannelStatus(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun bindAccount(call: MethodCall, result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.bindAccount(call.arguments as String?, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun unbindAccount(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.unbindAccount(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }

    //bindPhoneNumber


    private fun bindPhoneNumber(call: MethodCall, result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.bindPhoneNumber(call.arguments as String?, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun unbindPhoneNumber(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.unbindPhoneNumber(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun bindTag(call: MethodCall, result: Result) {
//        target: Int, tags: Array<String>, alias: String, callback: CommonCallback
        val target = call.argument("target") ?: 1
        val tagsInArrayList = call.argument("tags") ?: arrayListOf<String>()
        val alias = call.argument<String?>("alias")

        val arr = arrayOfNulls<String>(tagsInArrayList.size)
        val tags: Array<String> = tagsInArrayList.toArray(arr)

        val pushService = PushServiceFactory.getCloudPushService()

        pushService.bindTag(target, tags, alias, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun unbindTag(call: MethodCall, result: Result) {
//        target: Int, tags: Array<String>, alias: String, callback: CommonCallback
        val target = call.argument("target") ?: 1
        val tagsInArrayList = call.argument("tags") ?: arrayListOf<String>()
        val alias = call.argument<String?>("alias")

        val arr = arrayOfNulls<String>(tagsInArrayList.size)
        val tags: Array<String> = tagsInArrayList.toArray(arr)

        val pushService = PushServiceFactory.getCloudPushService()

        pushService.unbindTag(target, tags, alias, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }

    private fun listTags(call: MethodCall, result: Result) {
        val target = call.arguments as Int? ?: 1
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.listTags(target, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun addAlias(call: MethodCall, result: Result) {
        val alias = call.arguments as String?
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.addAlias(alias, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }

    private fun removeAlias(call: MethodCall, result: Result) {
        val alias = call.arguments as String?
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.removeAlias(alias, object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }

    private fun listAliases(result: Result) {
        val pushService = PushServiceFactory.getCloudPushService()
        pushService.listAliases(object : CommonCallback {
            override fun onSuccess(response: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to true,
                        "response" to response
                    )
                )

            }

            override fun onFailed(errorCode: String?, errorMessage: String?) {
                result.success(
                    mapOf(
                        "isSuccessful" to false,
                        "errorCode" to errorCode,
                        "errorMessage" to errorMessage
                    )
                )
            }
        })
    }


    private fun setupNotificationManager(call: MethodCall, result: Result) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channels = call.arguments as List<Map<String, Any?>>
            val mNotificationManager =
                gottenApplication.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            val notificationChannels = mutableListOf<NotificationChannel>()
            for (channel in channels) {
                // 通知渠道的id
                val id = channel["id"] ?: gottenApplication.packageName
                // 用户可以看到的通知渠道的名字.
                val name = channel["name"] ?: gottenApplication.packageName
                // 用户可以看到的通知渠道的描述
                val description = channel["description"] ?: gottenApplication.packageName
                val importance = channel["importance"] ?: NotificationManager.IMPORTANCE_DEFAULT
                val mChannel = NotificationChannel(id as String, name as String, importance as Int)
                // 配置通知渠道的属性
                mChannel.description = description as String
                mChannel.enableLights(true)
                mChannel.enableVibration(true)
                notificationChannels.add(mChannel)
            }
            if (notificationChannels.isNotEmpty()) {
                mNotificationManager.createNotificationChannels(notificationChannels)
            }
        }
        result.success(true)
    }

    //设置android角标
    private fun setApplicationBadgeNumber(call: MethodCall) {
        val appInfo = gottenApplication.packageManager
            .getApplicationInfo(gottenApplication.packageName, PackageManager.GET_META_DATA)

        val num = call.argument("num") as Int? ?: 0
        setHuaWeiApplicationBadgeNumber(num, appInfo)

    }

    //清理华为角标 https://developer.huawei.com/consumer/cn/doc/development/Corner-Guides/30802
    private fun setHuaWeiApplicationBadgeNumber(num: Int, appInfo: ApplicationInfo) {
        val huaweiAppId = appInfo.metaData.getString("com.huawei.hms.client.appid")
        if (huaweiAppId != null && huaweiAppId.toString().isNotBlank()) {
            try {
                val bundle = Bundle()
                bundle.putString(
                    "package",
                    gottenApplication.packageName
                ) // com.test.badge is your package name
                bundle.putString(
                    "class",
                    gottenApplication.packageName + ".MainActivity"
                ) // com.test. badge.MainActivity is your apk main activity
                bundle.putInt("badgenumber", num)
                gottenApplication.contentResolver.call(
                    Uri.parse("content://com.huawei.android.launcher.settings/badge/"),
                    "change_badge",
                    null,
                    bundle
                )

            } catch (e: Exception) {
                LogUtils.w("setHuaWeiApplicationBadgeNumberClean: 失败" + e.message)
            }
        }

    }

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {

        gottenApplication = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "com.jarvanmo/rammus")
        channel.setMethodCallHandler(this)

    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

}
